/* $Id: glthread.h,v 1.1 1999/12/16 17:31:06 brianp Exp $ */

/*
 * Mesa 3-D graphics library
 * Version:  3.3
 * 
 * Copyright (C) 1999  Brian Paul   All Rights Reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * BRIAN PAUL BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
 * AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */


/*
 * Thread support for gl dispatch.
 *
 * Initial version by John Stone (j.stone@acm.org) (johns@cs.umr.edu)
 *                and Christoph Poliwoda (poliwoda@volumegraphics.com)
 * Revised by Keith Whitwell
 * Adapted for new gl dispatcher by Brian Paul
 */



/*
 * If this file is accidentally included by a non-threaded build, 
 * it should not cause the build to fail, or otherwise cause problems.
 * In general, it should only be included when needed however.
 */
#ifdef THREADS
/*
 * It is an error not to select a specific threads API when compiling.
 */
#if !defined(PTHREADS) && !defined(SOLARIS_THREADS) && !defined(WIN32)
#error One of PTHREADS, SOLARIS_THREADS or WIN32 must be defined.
#endif



/*
 * POSIX threads. This should be your choice in the Unix world
 * whenever possible.  When building with POSIX threads, be sure
 * to enable any compiler flags which will cause the MT-safe
 * libc (if one exists) to be used when linking, as well as any
 * header macros for MT-safe errno, etc.  For Solaris, this is the -mt
 * compiler flag.  On Solaris with gcc, use -D_REENTRANT to enable
 * proper compiling for MT-safe libc etc.
 */
#ifdef PTHREADS
#include <pthread.h> /* POSIX threads headers */

typedef struct {
   pthread_key_t  key;
   pthread_once_t once;
} _glthread_TSD;

typedef pthread_mutex_t _glthread_Mutex;
typedef pthread_t _glthread_Thread;

#endif /* PTHREADS */




/*
 * Solaris threads. Use only up to Solaris 2.4. 
 * Solaris 2.5 and higher provide POSIX threads.
 * Be sure to compile with -mt on the Solaris compilers, or 
 * use -D_REENTRANT if using gcc.
 */
#ifdef SOLARIS_THREADS
#include <thread.h>

typedef struct {
   thread_key_t key;
   mutex_t      keylock;
   int          initfuncCalled;
} _glthread_TSD;

typedef mutex_t _glthread_Mutex;
typedef thread_t _glthread_Thread;

#endif /* SOLARIS_THREADS */




/*
 * Windows threads. Should work with Windows NT and 95. 
 * IMPORTANT: Link with multithreaded runtime library when THREADS are
 * used!
 */

#ifdef WIN32
#include <windows.h>

typedef struct {
   DWORD key;
   int   initfuncCalled;
} _glthread_TSD;

typedef CRITICAL_SECTION _glthread_Mutex;
typedef HANDLE _glthread_Thread;

#endif /* WIN32 */




/*
 * Platform independent thread specific data API. 
 */

extern unsigned long
_glthread_GetID(void);


extern void
_glthread_InitTSD(_glthread_TSD *);


extern void *
_glthread_GetTSD(_glthread_TSD *);


extern void
_glthread_SetTSD(_glthread_TSD *, void *, void (*initfunc)(void));



#endif
